/*
 *  Copyright (C) 2006-2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

/**
 * KibaObject:
 *
 * @KibaObject is the base class for the objects on @KibaDock.
 */

#ifndef __KIBA_OBJECT_H__
#define __KIBA_OBJECT_H__

#include <gtk/gtk.h>
#include <glib.h>
#include <cairo.h>

#include <kiba-title.h>
#include <kiba-progress.h>
#include <kiba-window.h>

G_BEGIN_DECLS

typedef struct _KibaObject         KibaObject;
typedef struct _KibaObjectClass    KibaObjectClass;

/**
 * KibaObjectPaintFunc:
 * @Param1: The KibaObject Instance.
 * @Param2: The drawing context.
 * @Param3: User Data.
 *
 * Method used for custom rendering.
 */
typedef void (*KibaObjectPaintFunc) (KibaObject*, cairo_t*, gpointer);

/**
 * KibaObjectCustomScaleFunc:
 * @Param1: The KibaObject Instance.
 * @Param2: The width scale factor return.
 * @Param3: The height scale factor return.
 * @Param4: User Data.
 *
 * Method used for custom scaling.
 */
typedef void (*KibaObjectCustomScaleFunc) (KibaObject*, float*, float*, gpointer);

/**
 * KibaObjectCustomMaxScaleFunc:
 * @Param1: The KibaObject Instance.
 * @Param2: The maximal width scale factor return.
 * @Param3: The maximal height scale factor return.
 * @Param4: User Data.
 *
 * Gets maximal scale factors for a scaling method.
 */
typedef void (*KibaObjectCustomMaxScaleFunc) (KibaObject*, float*, float*, gpointer);

/**
 * KibaObjectCustomOffsetFunc:
 * @Param1: The KibaObject Instance.
 * @Param2: The width offset return.
 * @Param3: The height offset return.
 * @Param4: User Data.
 *
 * Method used for custom surface offset.
 */
typedef void (*KibaObjectCustomOffsetFunc) (KibaObject*, float*, float*, gpointer);

/**
 * KibaObjectCustomMaxOffsetFunc:
 * @Param1: The KibaObject Instance.
 * @Param2: The maximal width offset return.
 * @Param3: The maximal height offset return.
 * @Param4: User Data.
 *
 * Method used for custom surface offset.
 */
typedef void (*KibaObjectCustomMaxOffsetFunc) (KibaObject*, float*, float*, gpointer);

/**
 * KibaObjectCustomRotateFunc:
 * @Param1: The KibaObject Instance.
 * @Param2: The rotate angle return.
 * @Param3: User Data.
 *
 * Method used for custom rotation.
 */
typedef void (*KibaObjectCustomRotateFunc) (KibaObject*, float*, gpointer);

#define KIBA_TYPE_OBJECT           (kiba_object_get_type ())
#define KIBA_OBJECT(obj)           (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_OBJECT, KibaObject))
#define KIBA_OBJECT_CLASS(obj)     (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_TYPE_OBJECT,  KibaObjectClass))
#define KIBA_IS_OBJECT(obj)        (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_OBJECT))
#define KIBA_IS_OBJECT_CLASS(obj)  (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_OBJECT))
#define KIBA_OBJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_OBJECT, KibaObjectClass))

/**
 * Object properties.
 **/
enum {
	KIBA_OBJECT_VISIBLE = 1,
	KIBA_OBJECT_MOVEABLE,
	KIBA_OBJECT_PAINTABLE,
	KIBA_OBJECT_PRELIGHTABLE,
	KIBA_OBJECT_ZOOMABLE,
	KIBA_OBJECT_REFLECTABLE,
	KIBA_OBJECT_HAS_SHADOW
};

/**
 * KibaObjectState:
 *
 * Object flags.
 */
typedef enum {
	KIBA_OBJECT_DRAGGING = 0,
	KIBA_OBJECT_PRELIGHTED,
	KIBA_OBJECT_URGENT,
	KIBA_OBJECT_ACTIVE,
	KIBA_OBJECT_FADE_IN,
	KIBA_OBJECT_FADE_OUT,
	KIBA_OBJECT_WORKING
}KibaObjectState;

typedef enum {
	KIBA_OBJECT_WINDOW_VISIBLE = 0,
	KIBA_OBJECT_WINDOW_INVISIBLE
}KibaObjectWindowState;

typedef enum {
	KIBA_OBJECT_ON_DOCK = 0,
	KIBA_OBJECT_LEFT_OF_DOCK,
	KIBA_OBJECT_RIGHT_OF_DOCK
}KibaObjectPosition;

/**
 * KibaObjectCustomScaleData:
 *
 * Type for custom scale animations.
 */
typedef struct {
	KibaObjectCustomScaleFunc scale_func;
	KibaObjectCustomMaxScaleFunc max_func;
	gboolean scale_widget;
	gpointer data;
}KibaObjectCustomScaleData;

/**
 * KibaObjectCustomPaintData:
 *
 * Type for custom rendering.
 */
typedef struct {
	KibaObjectPaintFunc func;
	gpointer data;
}KibaObjectCustomPaintData;

/**
 * KibaObjectCustomOffsetData:
 *
 * Type for custom surface offset animations.
 */
typedef struct {
	KibaObjectCustomScaleFunc place_func;
	KibaObjectCustomMaxScaleFunc max_func;
	gpointer data;
}KibaObjectCustomOffsetData;

/**
 * KibaObjectCustomRotateData:
 *
 * Type for custom rotate animations.
 */
typedef struct {
	KibaObjectCustomRotateFunc func;
	gpointer data;
}KibaObjectCustomRotateData;

struct _KibaObject {
	GtkBin bin;
	
	KibaObjectPosition position;
	
	KibaObjectWindowState win_state;
	KibaObjectState state;
	gint props;
	
	float alpha;            /**< current alpha value */
	float scroll_alpha;     /**< Alpha for the scroll fade */
	KibaRGBA shadow_color;
	gint shadow_affection;
	gint shadow_offset;
	
	KibaAllocation alloc;
	GtkRequisition size;
	float scale_x;        /**< widget width scale factor */
	float scale_y;        /**< widget height scale factor */
	float destroy_factor; /**< Scale factor for the destroy animation */
	
	
GtkAllocation reflection_allocation;
	
	KibaTitleObject *title;     /**< Pointer to KibaTitleObject instance */
	KibaProgressObject *progress;  /**< Pointer to KibaProgressObject instance */
	
	CairoSurfaceData *surface_data;   /**< The object surface */
	cairo_surface_t *switch_surface; /**< used for the switch animation */
	
	GList *custom_scale_funcs; /**< list of all KibaObjectCustomScaleFunc's */
	GList *custom_offset_data; /**< list of all KibaObjectCustomOffsetData's */
	GList *custom_rotate_funcs; /**< list of all KibaObjectCustomRotateData's */
	
	KibaObjectPaintFunc paint;                     /**< the paint function */
	gpointer paint_data;                /**< the paint data */
	GList *custom_paint_funcs_after;  /**< custom paint methods before the paint method */
	GList *custom_paint_funcs_before; /**< custom paint methods after the paint method */
};

struct _KibaObjectClass {
	GtkBinClass parent_class;
	
	/* signals */
	void (* switch_event) (KibaObject *object);
	void (* urgent_toggled) (KibaObject  *object,
				 gboolean     urgent);
	void (* active_toggled) (KibaObject  *object,
				 gboolean     active);
};

GtkWidget* kiba_object_new (const char *group);

int kiba_object_launch(KibaObject *object,
                        const char *command);

gboolean kiba_object_update_scale_factor (KibaObject *object);

void kiba_object_set_requisition_factor (KibaObject *object,
					 float      scale_x,
					 float      scale_y);

void kiba_object_set_alpha (KibaObject *object,
			    float      alpha);

void kiba_object_set_icon_geometry (KibaObject *object,
				    KibaWindow  *window);

void kiba_object_queue_draw (KibaObject *object);

void kiba_object_paint (KibaObject *object,
		        cairo_t    *cr,
		        gpointer    data);

void kiba_object_paint_reflection (KibaObject *object,
				   cairo_t    *cr);

void kiba_object_paint_gear (KibaObject *object,
                             cairo_t *cr);

void kiba_object_paint_shadow (KibaObject *object,
			       cairo_t    *cr);

void kiba_object_paint_with_prelight (KibaObject       *object,
				      cairo_surface_t  *surface,
				      cairo_t          *cr);

void kiba_object_rotate (KibaObject *object,
			 cairo_t    *cr);

gboolean kiba_object_scale (KibaObject *object,
			    cairo_t    *cr);

void kiba_object_switch_surface (KibaObject *object);

void kiba_object_set_paint_function (KibaObject          *object,
				     KibaObjectPaintFunc  func,
				     gpointer             data);

void kiba_object_add_custom_rotate_function (KibaObject                 *object,
					     KibaObjectCustomRotateFunc  func,
					     gpointer                    user_data);

void kiba_object_remove_custom_rotate_function_by_data (KibaObject *object,
						        gpointer    user_data);

void kiba_object_add_custom_offset_function (KibaObject                  *object,
					     KibaObjectCustomOffsetFunc   place_func,
					     KibaObjectCustomMaxScaleFunc max_func,
					     gpointer                     user_data);

void kiba_object_remove_custom_offset_function_by_data (KibaObject *object,
						        gpointer    user_data);

void kiba_object_add_custom_paint_function (KibaObject           *object,
					    KibaObjectPaintFunc   func,
					    gboolean              before,
					    gpointer              user_data);

void kiba_object_remove_custom_paint_function_by_data (KibaObject *object,
						       gpointer    user_data);

void kiba_object_add_custom_scale_function (KibaObject                   *object,
					    KibaObjectCustomScaleFunc     scale_func,
					    KibaObjectCustomMaxScaleFunc  max_func,
					    gboolean                      scale_widget,
					    gpointer                      user_data);

void kiba_object_remove_custom_scale_function_by_data (KibaObject *object,
						       gpointer    user_data);

void kiba_object_set_urgent (KibaObject *object,
			     gboolean    urgent);

void kiba_object_set_working (KibaObject *object,
                              gboolean    workDone);

gboolean kiba_object_get_urgent (KibaObject *object);

void kiba_object_set_reflectable (KibaObject *object,
				  gboolean    reflect);

gboolean kiba_object_get_reflectable (KibaObject *object);

float kiba_object_get_destroy_factor (KibaObject *object);

gboolean kiba_object_get_destroying (KibaObject *object);

void kiba_object_set_has_shadow (KibaObject *object,
				 gboolean    has_shadow);

gboolean kiba_object_get_has_shadow (KibaObject *object);

void kiba_object_set_title (KibaObject *object,
                            const char *title);

const char* kiba_object_get_title (KibaObject *object);

void kiba_object_set_tooltip_text(KibaObject *object,
                                  const char *text);

void kiba_object_show_title (GtkWidget *widget);

void kiba_object_hide_title (GtkWidget *widget);

void kiba_object_set_active (GtkWidget *widget,
                             gboolean   active);

gboolean kiba_object_get_active (GtkWidget *widget);

void kiba_object_set_prelight (GtkWidget *widget);

void kiba_object_unset_prelight (GtkWidget *widget);

gboolean kiba_object_is_on_dock (KibaObject *object);

void kiba_object_destroy (KibaObject *object);

void kiba_object_resize (KibaObject *object);


G_END_DECLS

#endif
