/*
 *  Copyright (C) 2006-2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

#ifndef __KIBA_PREFS_H__
#define __KIBA_PREFS_H__

#include <gtk/gtk.h>
#include <glib.h>
#include <glib-object.h>

#include <kiba-types.h>

G_BEGIN_DECLS

/**
 * KibaPrefs is a settings database.
 **/

typedef struct _KibaPrefs      KibaPrefs;
typedef struct _KibaPrefGroup  KibaPrefGroup;
typedef struct _KibaPrefOption KibaPrefOption;
typedef struct _PrefMember     PrefMember;
typedef struct _KibaTypeData   KibaTypeData;

/* setting changed callback type */
typedef void (*KibaPrefCallback) (const char*, gpointer, gpointer);

typedef enum {
	KIBA_INT = 0,
	KIBA_BOOLEAN,
	KIBA_DOUBLE,
	KIBA_STRING,
	KIBA_LIST,
	KIBA_INVALID
}KibaType;

static const struct { KibaType type; const char *pref_type; } kiba_type_data[] = {
	{ KIBA_INT, "int" },
	{ KIBA_BOOLEAN, "bool" },
	{ KIBA_DOUBLE, "double" },
	{ KIBA_STRING, "char" },
	{ KIBA_LIST, "list" },
	{ KIBA_INVALID, "invalid" }
};

/* type used for initialling settings in the database */
struct _PrefMember {
	const char    *id;             /**< setting name/id */
	const char    *default_value;  /**< default value   */
	KibaType       type;           /**< value type      */
};

struct _KibaPrefOption {
	PrefMember   member;     /**< Array of PrefMember's */
	gpointer     value;	 /**< Value pointer */
	GList       *callbacks;  /**< List of callbacks for the setting */
};

struct _KibaPrefGroup {
	PrefMember  *member;          /**< Array of PrefMember's */
	gint         member_elements; /**< Number of member elements */
	GList       *options;         /**< List with all KibaPrefOption's */
};

struct _KibaPrefs {
	char *config_file; /**< The path to the config file */
	GList *groups; /**< List with all KibaPrefGroup's */
	GList *options; /**< list of KibaOptions's from the config file **/
};

typedef struct {
	char *id;
	char *type;
	char *value;
}KibaOption;

void kiba_prefs_init (const char *config_file);

KibaPrefs* kiba_prefs_get (void);

void kiba_prefs_add_callback (const char       *id,
			      KibaPrefCallback  callback,
			      gpointer          data,
			      gboolean          launch);

void kiba_prefs_remove_callback_by_data (gpointer data);

gint kiba_prefs_edge_style_from_string (const char *string);

gint kiba_prefs_background_style_from_string (const char *string);

KibaRGBA kiba_prefs_rgba_from_string (const char *string);

KibaPosition kiba_prefs_get_position_from_string (const char *string);

KibaType kiba_prefs_get_type_from_string (const char *string);

gpointer kiba_prefs_get_pointer (const char *id);

float kiba_prefs_get_float (const char *id);

gboolean kiba_prefs_get_bool (const char *id);

gint kiba_prefs_get_int (const char *id);

gchar* kiba_prefs_get_string (const char *id);

GSList* kiba_prefs_get_list (const char *id);

void kiba_prefs_append_to_list (const char *list_id,
                                const char *new_item);

void kiba_prefs_remove_from_list (const char *list_id,
                                  const char *remove_item);

GSList* kiba_prefs_get_list_from_string (const char *data);

char* kiba_prefs_list_to_string(GList *list);

void kiba_prefs_set_option (const char *id,
			    const char *type,
			    const char *value);

void kiba_prefs_remove_options (PrefMember *options);

void kiba_prefs_add_options (PrefMember *options,
			     gint        option_elements);

const PrefMember* kiba_prefs_get_member_for_id (const PrefMember options[],
						int              array_elemets,
						const char      *setting_id);

char* kiba_prefs_get_option_from_file (const char *id,
				       const char *type);

gboolean kiba_prefs_write_to_config_file (const char *id,
					  const char *type,
					  const char *value);

void kiba_prefs_update_config_file(KibaPrefs *prefs);

void kiba_prefs_fini (void);

G_END_DECLS

#endif
